<?php
/******************************************************************************\
**   JoomGallery 3 Migration script for ICE Gallery                           **
**   By: JoomGallery::ProjectTeam                                             **
**   Copyright (C) 2019 JoomGallery::ProjectTeam                              **
**   Based on: JoomGallery 1.0.0 by JoomGallery::ProjectTeam                  **
**   Released under GNU GPL Public License                                    **
**   License: http://www.gnu.org/copyleft/gpl.html or have a look             **
**   at administrator/components/com_joomgallery/LICENSE.TXT                  **
\******************************************************************************/

/*******************************************************************************
**   Migration of DB and Files from ICE gallery to Joomgallery                **
**   On the fly generating of categories in db and file system                **
**   moving the pictures in the new categories                                **
**   moving the pictures in the new categories                                **
*******************************************************************************/

defined('_JEXEC') or die('Direct Access to this location is not allowed.');

class JoomMigrateice2jux extends JoomMigration
{
  /**
   * The name of the migration
   * (should be unique)
   *
   * @var string
   */
  protected $migration = 'ice2jux';

  /**
   * Properties for paths and database table names of old JoomGallery to migrate from
   *
   * @var string
   */
  protected $prefix;
  protected $path;
  protected $path_originals;
  protected $table_images;
  protected $table_categories;
  protected $table_comments;
//  protected $table_users;

  /**
   * Constructor
   *
   * @return  void
   * @since   1.0.0
   */
  public function __construct()
  {
    parent::__construct();

    $prefix = $this->getStateFromRequest('prefix', 'prefix', '', 'cmd');
    $path   = $this->getStateFromRequest('path', 'path', '', 'string');

    $this->prefix             = $prefix;
    $this->path_originals     = $path.'/';
    $this->path_details       = $path.'/';
    $this->path_thumbnails    = $path.'/';
    $this->table_images       = $prefix.'icefiles';
    $this->table_categories   = $prefix.'ice';
    $this->table_comments     = $prefix.'ice_comments';

  }

  /**
   * Checks requirements for migration
   *
   * @return  void
   * @since   1.0.0
   */
  public function check($dirs = array(), $tables = array(), $xml = false, $min_version = false, $max_version = false)
  {

    if($this->path == JPATH_ROOT)
    {
      JFactory::getLanguage()->load('com_joomgallery.migrateice2jux');
      $this->_mainframe->redirect('index.php?option='._JOOM_OPTION.'&controller=migration', JText::_('FILES_JOOMGALLERY_MIGRATION_ICE2JUX_WRONG_PATH2ICE'), 'notice');
    }

/*    if(!$this->otherDatabase && $this->prefix == $this->_db->getPrefix())
    {
      JFactory::getLanguage()->load('com_joomgallery.migrateice2jux');
      $this->_mainframe->redirect('index.php?option='._JOOM_OPTION.'&controller=migration', JText::_('FILES_JOOMGALLERY_MIGRATION_ICE2JUX_WRONG_PREFIX'), 'notice');
    }
*/

    $tables = array($this->table_images,
                    $this->table_categories,
                    $this->table_comments);
        
    $dirs = array($this->path_originals,
                  $this->path_details,
                  $this->path_thumbnails);

    parent::check($dirs, $tables, $xml, $min_version, $max_version);
  }

  /**
   * Main migration function
   *
   * @return  void
   * @since   1.0.0
   */
  protected function doMigration()
  {
    $task = $this->getTask('categories');

    switch($task)
    {
      case 'categories':
        $this->migrateCategories();
        // Break intentionally omited
      case 'rebuild':
        $this->rebuild();
        // Break intentionally omited
      case 'images':
        $this->migrateImages();
        // Break intentionally omited
      case 'comments':
        $this->migrateComments();
        // Break intentionally omited
      default:
        break;
    }
  }

  /**
   * Returns the maximum category ID of ICE gallery.
   *
   * @return  int   The maximum category ID of ICE gallery
   * @since   1.0.0
   */
  protected function getMaxCategoryId()
  {
    $query = $this->_db2->getQuery(true)
          ->select('MAX(catid)')
          ->from($this->table_categories);
    $this->_db2->setQuery($query);

    return $this->runQuery('loadResult', $this->_db2);
  }

  /**
   * Returns the maximum pos ordering number of ICE gallery.
   *
   * @return  int   The maximum pos number of ICE gallery
   * @since   1.0.0
   */
  protected function getMaxPos()
  {
    $query = $this->_db2->getQuery(true)
          ->select('MAX(pos)')
          ->from($this->table_categories);
    $this->_db2->setQuery($query);

    return $this->runQuery('loadResult', $this->_db2);
  }

  /**
   * Migrates all public categories
   *
   * @return  void
   * @since   1.0.0
   */
  protected function migrateCategories()
  {
    $this->writeLogfile('Start migrating categories');

    $query = $this->_db2->getQuery(true)
          ->select('*')
          ->from($this->table_categories);
    $this->prepareTable($query, $this->table_categories, 'subcat_id', array(0));

    $maxpos = $this->getMaxPos();

    while($cat = $this->getNextObject())
    {
      // Make information accessible for JoomGallery
      $cat->cid          = $cat->catid;
      $cat->name         = $cat->catname;
      $cat->description  = $cat->catdescr;
      $cat->parent_id    = $cat->subcat_id;
      $cat->published    = $cat->published;
      $cat->catpath      = $cat->catdir;
      $cat->owner        = $cat->uid;
      $cat->metakey      = $cat->metakey;
      $cat->metadesc     = $cat->metadescr;
      $cat->thumbnail    = $cat->catimg;
      $cat->lft          = $maxpos - $cat->pos;
      $cat->access       = intval(substr($cat->catmembers, 0, strpos($cat->catmembers, ';')));

      $this->createCategory($cat);

      $this->markAsMigrated($cat->catid, 'catid', $this->table_categories);

      if(!$this->checkTime())
      {
        $this->refresh();
      }
    }

    $this->resetTable($this->table_categories);

    $this->writeLogfile('Categories successfully migrated');
  }


  /**
   * Migrates all public images
   *
   * @return  void
   * @since   1.0.0
   */
  protected function migrateImages()
  {
    $this->writeLogfile('Start migrating images');

    $query = $this->_db2->getQuery(true)
          ->select('*')
          ->from($this->table_images)
          ->innerJoin($this->table_categories.' ON '.$this->prefix.'icefiles.catid = '.$this->prefix.'ice.catid');
    $this->prepareTable($query);

    while($row = $this->getNextObject())
    {
      $original   = $this->path_originals.$row->catdir.'/'.$row->imgfilename;
      
      if(!JFile::exists($original))
      {
        if(!JFile::exists($original))
        {
          $this->setError('Original-Image not found: '.$original);

          continue;
        }
      }

      $row->id           = $row->imgid;
      $row->catid        = $row->catid;
      $row->imgtitle     = $row->imgname;
      $row->imgtext      = $row->imgdescr;
      $row->imgdate      = $row->imgdate;
      $row->published    = $row->published;
      $row->imgfilename  = $row->imgfilename;
      $row->imgvotes     = $row->votenum;
      $row->imgvotesum   = $row->votesum;
      $row->hits         = $row->imghits;
      $row->owner        = $row->uid;
      $row->ordering     = $row->pos;
      $row->downloads    = $row->downloads;
      $row->alias        = '';
      $row->access       = intval(substr($row->imgmembers, 0, strpos($row->imgmembers, ';')));

      $this->moveAndResizeImage($row, $original, null, null, true);

      if(!$this->checkTime())
      {
        $this->refresh('images');
      }
    }

    $this->resetTable();

    $this->writeLogfile('Images successfully migrated');
  }


  /**
   * Migrate all comments
   *
   * @return  void
   * @since   1.0.0
   */
  protected function migrateComments()
  {
    $this->writeLogfile('Start migrating comments');

    $query = $this->_db2->getQuery(true)
          ->select('*')
          ->from($this->table_comments);
    $this->prepareTable($query);

    while($row = $this->getNextObject())
    {
      $row->cmtid       = $row->cmtid;
      $row->cmtpic      = $row->imgid;
      $row->cmtname     = $row->cmtname;
      $row->cmttext     = $row->cmtcontent;
      $row->cmtip       = $row->hostaddr;
      $row->published   = $row->published;
      $row->cmtdate     = $row->cmtdate;

      $this->createComment($row);

      if(!$this->checkTime())
      {
        $this->refresh('comments');
      }
    }

    $this->resetTable();

    $this->writeLogfile('Comments successfully migrated');
  }

  /**
   * Migrates configuration settings (where possible)
   *
   * @return  void
   * @since   1.0
   */
  protected function migrateConfig()                         // not possible
  {
   
  }

  /**
   * Migrates image details (additional fields)
   *
   * @return  void
   * @since   1.0
   */
  protected function migrateImageDetails()                    // not possible
  {
    
  }

  /**
   * Migrates category details (additional fields)
   *
   * @return  void
   * @since   1.0
   */
  protected function migrateCategoryDetails()                 // not possible
  {
    
  }

}
